<?php

namespace Drupal\media_canonical_download\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Field\BaseFieldDefinition;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Returns responses for COS Media as File routes.
 */
class MediaCanonicalDownloadController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->configFactory = $container->get('config.factory');
    $instance->moduleHandler = $container->get('module_handler');
    $instance->currentUser = $container->get('current_user');

    return $instance;
  }

  /**
   * Builds the response.
   */
  public function build(MediaInterface $media = NULL, Request $request = NULL) {
    $config = $this->configFactory->get('media_canonical_download.settings');
    $settings = $config->get($media->bundle());
    $enabled = $settings['enabled'] ?? FALSE;
    $overrides = $settings['overrides'] ?? [];
    $ret = [];

    foreach ($media->getFieldDefinitions() as $field_name => $definition) {
      if ($field_name != 'thumbnail' && !($definition instanceof BaseFieldDefinition) && ($field = $media->get($field_name)) instanceof FileFieldItemList) {
        $file = $field->entity ?? NULL;
        $uri = ($file->uri->value ?? NULL);

        if ($file) {
          $mime = $file->getMimeType();
          $ext = pathinfo($uri, PATHINFO_EXTENSION);

          if ($enabled xor (($ext && in_array($ext, $overrides)) || ($mime && in_array($mime, $overrides)))) {
            $ret = [
              'uri' => $uri,
              'status' => 200,
              'headers' => [
                'Content-Type' => $file->getMimeType(),
                'Content-Length' => $file->getSize(),
                'Cache-Control' => 'private',
              ],
            ];

            break;
          }
        }
      }
    }

    $this->moduleHandler()->alter('media_canonical_download', $ret, $media, $this->currentUser, $request);

    if ($ret instanceof Response) {
      return $ret;
    }

    if (!empty($ret) && !empty($ret['uri'] ?? NULL)) {
      if (!file_exists($ret['uri'])) {
        return new Response('Expected file does not exist.', 500);
      }

      return new BinaryFileResponse($ret['uri'], $ret['status'] ?? NULL, $ret['headers'] ?? []);
    }

    return $this->entityTypeManager->getViewBuilder('media')->view($media);
  }

}
